@@ -29,8 +29,7 @@ __PACKAGE__->mk_attr_accessors(qw/ id /);
 
 __PACKAGE__->mk_accessors(
     qw/
-        name type filename is_field is_repeatable db
-        /
+        name type filename is_field is_repeatable /
 );
 
 __PACKAGE__->mk_inherited_accessors(qw/ tt_args render_method /);
@@ -46,9 +45,8 @@ sub new {
 
     my $self = bless {}, $class;
 
-    $self->attributes( {} );
-    $self->stash(      {} );
-    $self->db(         {} );
+    $self->attributes(  {} );
+    $self->stash(       {} );
 
     if ( exists $attrs{parent} ) {
         $self->parent( delete $attrs{parent} );
@@ -59,6 +57,28 @@ sub new {
     return $self;
 }
 
+sub db {
+    my $self = shift;
+
+    warn "db() method deprecated and is provided for compatibilty only: "
+        . "use model_config() instead as this will be removed\n";
+
+    if (@_) {
+        my $args = shift;
+
+        $self->model_config = {}
+            if !$self->model_config;
+
+        my $conf = $self->model_config;
+
+        for ( keys %$args ) {
+            $conf->{$_} = $args->{$_};
+        }
+    }
+
+    return $self->model_config;
+}
+
 sub setup { }
 
 sub get_elements { [] }
@@ -91,6 +111,8 @@ sub clear_errors { }
 
 sub process { }
 
+sub post_process { }
+
 sub prepare_id { }
 
 sub prepare_attrs { }
@@ -115,7 +137,8 @@ sub clone {
     $new{tt_args} = dclone $self->{tt_args}
         if $self->{tt_args};
 
-    $new{attributes} = dclone $self->attributes;
+    $new{attributes}  = dclone $self->attributes;
+    $new{model_config} = dclone $self->model_config;
 
     return bless \%new, ref $self;
 }
@@ -339,6 +362,14 @@ Get or set the element's DOM id.
 
 Default Value: none
 
+=head1 MODEL / DATABASE INTERACTION
+
+See L<HTML::FormFu::Model> for further details and available models.
+
+=head2 model_config
+
+Arguments: \%config
+
 =head1 RENDERING
 
 =head2 filename
@@ -465,6 +496,15 @@ used directly.
 
 =back
 
+=head1 DEPRECATED METHODS
+
+=head2 db
+
+Is deprecated and provided only for backwards compatability. Will be removed
+at some point in the future.
+
+Use L</model_config> instead.
+
 =head1 AUTHOR
 
 Carl Franks, C<cfranks@cpan.org>
