@@ -9,15 +9,20 @@ use HTML::FormFu::ObjectUtil qw/
     get_error _require_constraint set_nested_hash_value nested_hash_key_exists
     get_nested_hash_value /;
 use HTML::FormFu::Util qw/
-    _parse_args append_xml_attribute xml_escape require_class process_attrs /;
+    _parse_args append_xml_attribute xml_escape require_class process_attrs
+    _filter_components /;
+use List::MoreUtils qw/ uniq /;
 use Storable qw/ dclone /;
 use Carp qw/ croak /;
 use Exporter qw/ import /;
 
 # used by multi.pm
 our @EXPORT_OK = qw/
-    _render_container_class _render_comment_class _render_label
-    _string_field_start _string_field_end _string_label /;
+    nested_name add_error 
+    deflator filter constraint inflator validator transformer plugin
+    deflators filters constraints inflators validators transformers plugins /;
+
+our %EXPORT_TAGS = ( FIELD => \@EXPORT_OK );
 
 __PACKAGE__->mk_attrs(
     qw/
@@ -30,7 +35,7 @@ __PACKAGE__->mk_attrs(
 __PACKAGE__->mk_accessors(
     qw/
         _constraints _filters _inflators _deflators _validators _transformers
-        _errors container_tag
+        _plugins _errors container_tag
         field_filename label_filename label_tag retain_default force_default
         javascript non_param reverse_multi multi_value original_name /
 );
@@ -63,13 +68,14 @@ sub new {
     $self->_inflators(    [] );
     $self->_validators(   [] );
     $self->_transformers( [] );
+    $self->_plugins(      [] );
     $self->_errors(       [] );
     $self->comment_attributes(   {} );
     $self->container_attributes( {} );
     $self->label_attributes(     {} );
     $self->label_filename('label');
     $self->label_tag('label');
-    $self->container_tag('span');
+    $self->container_tag('div');
     $self->is_field(1);
 
     return $self;
@@ -243,21 +249,27 @@ sub transformer {
     return @return == 1 ? $return[0] : @return;
 }
 
+sub plugin {
+    my ( $self, $arg ) = @_;
+    my @return;
+
+    if ( ref $arg eq 'ARRAY' ) {
+        push @return, map { _single_plugin( $self, $_) } @$arg;
+    }
+    else {
+        push @return, _single_plugin( $self, $arg );
+    }
+
+    return @return == 1 ? $return[0] : @return;
+}
+
 sub get_deflators {
     my $self = shift;
     my %args = _parse_args(@_);
 
     my @x = @{ $self->_deflators };
 
-    if ( exists $args{name} ) {
-        @x = grep { $_->name eq $args{name} } @x;
-    }
-
-    if ( exists $args{type} ) {
-        @x = grep { $_->type eq $args{type} } @x;
-    }
-
-    return \@x;
+    return _filter_components( \%args, \@x );
 }
 
 sub get_filters {
@@ -266,15 +278,7 @@ sub get_filters {
 
     my @x = @{ $self->_filters };
 
-    if ( exists $args{name} ) {
-        @x = grep { $_->name eq $args{name} } @x;
-    }
-
-    if ( exists $args{type} ) {
-        @x = grep { $_->type eq $args{type} } @x;
-    }
-
-    return \@x;
+    return _filter_components( \%args, \@x );
 }
 
 sub get_constraints {
@@ -283,15 +287,7 @@ sub get_constraints {
 
     my @x = @{ $self->_constraints };
 
-    if ( exists $args{name} ) {
-        @x = grep { $_->name eq $args{name} } @x;
-    }
-
-    if ( exists $args{type} ) {
-        @x = grep { $_->type eq $args{type} } @x;
-    }
-
-    return \@x;
+    return _filter_components( \%args, \@x );
 }
 
 sub get_inflators {
@@ -300,15 +296,7 @@ sub get_inflators {
 
     my @x = @{ $self->_inflators };
 
-    if ( exists $args{name} ) {
-        @x = grep { $_->name eq $args{name} } @x;
-    }
-
-    if ( exists $args{type} ) {
-        @x = grep { $_->type eq $args{type} } @x;
-    }
-
-    return \@x;
+    return _filter_components( \%args, \@x );
 }
 
 sub get_validators {
@@ -317,15 +305,7 @@ sub get_validators {
 
     my @x = @{ $self->_validators };
 
-    if ( exists $args{name} ) {
-        @x = grep { $_->name eq $args{name} } @x;
-    }
-
-    if ( exists $args{type} ) {
-        @x = grep { $_->type eq $args{type} } @x;
-    }
-
-    return \@x;
+    return _filter_components( \%args, \@x );
 }
 
 sub get_transformers {
@@ -334,40 +314,22 @@ sub get_transformers {
 
     my @x = @{ $self->_transformers };
 
-    if ( exists $args{name} ) {
-        @x = grep { $_->name eq $args{name} } @x;
-    }
-
-    if ( exists $args{type} ) {
-        @x = grep { $_->type eq $args{type} } @x;
-    }
-
-    return \@x;
+    return _filter_components( \%args, \@x );
 }
 
 sub get_errors {
     my $self = shift;
     my %args = _parse_args(@_);
 
-    my @e = @{ $self->_errors };
+    my @x = @{ $self->_errors };
 
-    if ( exists $args{name} ) {
-        @e = grep { $_->name eq $args{name} } @e;
-    }
-
-    if ( exists $args{type} ) {
-        @e = grep { $_->type eq $args{type} } @e;
-    }
-
-    if ( exists $args{stage} ) {
-        @e = grep { $_->stage eq $args{stage} } @e;
-    }
+    _filter_components( \%args, \@x );
 
     if ( !$args{forced} ) {
-        @e = grep { !$_->forced } @e;
+        @x = grep { !$_->forced } @x;
     }
 
-    return \@e;
+    return \@x;
 }
 
 sub add_error {
@@ -386,6 +348,30 @@ sub clear_errors {
     return;
 }
 
+sub process {
+    my $self = shift;
+
+    $self->next::method(@_);
+
+    for my $plugin ( @{ $self->_plugins } ) {
+        $plugin->process;
+    }
+
+    return;
+}
+
+sub post_process {
+    my $self = shift;
+
+    $self->next::method(@_);
+
+    for my $plugin ( @{ $self->_plugins } ) {
+        $plugin->post_process;
+    }
+
+    return;
+}
+
 sub process_input {
     my ( $self, $input ) = @_;
 
@@ -428,8 +414,8 @@ sub prepare_id {
         && length $self->auto_id )
     {
         my %string = (
-            f => defined $self->form->id ? $self->form->id : '',
-            n => defined $render->{name} ? $render->{name} : '',
+            f => defined $self->form->id          ? $self->form->id          : '',
+            n => defined $render->{ nested_name } ? $render->{ nested_name } : '',
         );
 
         my $id = $self->auto_id;
@@ -567,15 +553,13 @@ sub _render_value {
 
     my $form = $self->form;
     my $name = $self->nested_name;
-    my $render_processed;
 
     my $input
         = (    $self->form->submitted
             && defined $name
             && $self->nested_hash_key_exists( $form->input, $name ) )
         ? $self->render_processed_value
-            ? ( $render_processed = 1
-                    && $self->get_nested_hash_value(
+            ? ( $self->get_nested_hash_value(
                     $form->_processed_params, $name
                     ) )
             : $self->get_nested_hash_value( $form->input, $name )
@@ -592,7 +576,7 @@ sub _render_value {
 
     my $value = $self->process_value($input);
 
-    if ( !$self->form->submitted || $render_processed ) {
+    if ( !$self->form->submitted || ( $self->render_processed_value && defined $value ) ) {
         for my $deflator ( @{ $self->_deflators } ) {
             $value = $deflator->process($value);
         }
@@ -749,9 +733,11 @@ sub _render_error_class {
         append_xml_attribute( $render->{container_attributes},
             'class', 'error' );
 
-        for my $error (@errors) {
+        my @class = uniq sort map { $_->class } @errors;
+
+        for my $class (@class) {
             append_xml_attribute( $render->{container_attributes},
-                'class', $error->class );
+                'class', $class );
         }
     }
 
@@ -873,7 +859,10 @@ sub _single_deflator {
     if ( !ref $arg ) {
         $arg = { type => $arg };
     }
-    elsif ( ref $arg ne 'HASH' ) {
+    elsif ( ref $arg eq 'HASH' ) {
+        $arg = { %$arg }; # shallow clone
+    }
+    else {
         croak 'invalid args';
     }
 
@@ -894,7 +883,10 @@ sub _single_filter {
     if ( !ref $arg ) {
         $arg = { type => $arg };
     }
-    elsif ( ref $arg ne 'HASH' ) {
+    elsif ( ref $arg eq 'HASH' ) {
+        $arg = { %$arg }; # shallow clone
+    }
+    else {
         croak 'invalid args';
     }
 
@@ -915,7 +907,10 @@ sub _single_constraint {
     if ( !ref $arg ) {
         $arg = { type => $arg };
     }
-    elsif ( ref $arg ne 'HASH' ) {
+    elsif ( ref $arg eq 'HASH' ) {
+        $arg = { %$arg }; # shallow clone
+    }
+    else {
         croak 'invalid args';
     }
 
@@ -936,7 +931,10 @@ sub _single_inflator {
     if ( !ref $arg ) {
         $arg = { type => $arg };
     }
-    elsif ( ref $arg ne 'HASH' ) {
+    elsif ( ref $arg eq 'HASH' ) {
+        $arg = { %$arg }; # shallow clone
+    }
+    else {
         croak 'invalid args';
     }
 
@@ -957,7 +955,10 @@ sub _single_validator {
     if ( !ref $arg ) {
         $arg = { type => $arg };
     }
-    elsif ( ref $arg ne 'HASH' ) {
+    elsif ( ref $arg eq 'HASH' ) {
+        $arg = { %$arg }; # shallow clone
+    }
+    else {
         croak 'invalid args';
     }
 
@@ -978,7 +979,10 @@ sub _single_transformer {
     if ( !ref $arg ) {
         $arg = { type => $arg };
     }
-    elsif ( ref $arg ne 'HASH' ) {
+    elsif ( ref $arg eq 'HASH' ) {
+        $arg = { %$arg }; # shallow clone
+    }
+    else {
         croak 'invalid args';
     }
 
@@ -993,6 +997,30 @@ sub _single_transformer {
     return $new;
 }
 
+sub _single_plugin {
+    my ( $self, $arg ) = @_;
+
+    if ( !ref $arg ) {
+        $arg = { type => $arg };
+    }
+    elsif ( ref $arg eq 'HASH' ) {
+        $arg = { %$arg }; # shallow clone
+    }
+    else {
+        croak 'invalid args';
+    }
+
+    my @return;
+
+    my $type = delete $arg->{type};
+
+    my $new = $self->_require_plugin( $type, $arg );
+
+    push @{ $self->_plugins }, $new;
+
+    return $new;
+}
+
 1;
 
 __END__
@@ -1104,7 +1132,7 @@ Set the comment using a L10N key.
 Set which tag-name should be used to contain the various field parts (field, 
 label, comment, errors, etc.).
 
-Default Value: 'span'
+Default Value: 'div'
 
 =head2 javascript
 
