@@ -5,16 +5,22 @@ use base 'HTML::FormFu::Element::_Field';
 use Class::C3;
 
 use HTML::FormFu::ObjectUtil qw/ _coerce /;
-use HTML::FormFu::Util qw/ append_xml_attribute literal /;
+use HTML::FormFu::Util qw/ append_xml_attribute literal xml_escape /;
+use HTML::FormFu::Attribute qw/ mk_output_accessors /;
 use Storable qw( dclone );
 use Carp qw( croak );
 
 __PACKAGE__->mk_accessors(qw/ _options empty_first /);
+__PACKAGE__->mk_output_accessors(qw/ empty_first_label/);
 
 my @ALLOWED_OPTION_KEYS = qw/
     group
     value
+    value_xml
+    value_loc
     label
+    label_xml
+    label_loc
     attributes
     attrs
     attributes_xml
@@ -35,20 +41,35 @@ sub new {
 }
 
 sub process {
-    my ($self) = @_;
+    my $self = shift;
 
-    my $args = $self->db;
+    $self->next::method(@_);
 
-    if ( $args and keys %$args ) {
-        $self->options(
-            [ $self->form->model->options_from_model( $self, $args ) ] );
-    }
+    my $args = $self->model_config;
+
+    return unless $args && keys %$args;
+
+    return if @{ $self->options };
+
+    # don't run if {options_from_model} is set and is 0
+
+    my $option_flag = exists $args->{options_from_model}
+        ? $args->{options_from_model}
+        : 1;
+
+    return if !$option_flag;
+    
+    $self->options( [ $self->form->model->options_from_model( $self, $args ) ] );
+    
+    return;
 }
 
 sub options {
     my ( $self, $arg ) = @_;
     my ( @options, @new );
 
+    return $self->_options if @_ == 1;
+
     croak "options argument must be a single array-ref" if @_ > 2;
 
     if ( defined $arg ) {
@@ -56,13 +77,7 @@ sub options {
         croak "options argument must be an array-ref" if $@;
 
         if ( $self->empty_first ) {
-            push @new,
-                {
-                value            => '',
-                label            => '',
-                attributes       => {},
-                label_attributes => {},
-                };
+            push @new, $self->_get_empty_first_option;
         }
 
         for my $item (@options) {
@@ -75,6 +90,19 @@ sub options {
     return $self;
 }
 
+sub _get_empty_first_option {
+    my ($self) = @_;
+
+    my $l = $self->empty_first_label || '';
+
+    return {
+        value            => '',
+        label            => $l,
+        attributes       => {},
+        label_attributes => {},
+    };
+}
+
 sub _parse_option {
     my ( $self, $item ) = @_;
 
@@ -130,13 +158,7 @@ sub _parse_option_hashref {
         for my $groupitem (@group) {
             push @new, $self->_parse_option($groupitem);
         }
-        my %group = ( group => \@new );
-        $group{label} = $item->{label};
-        $group{label} = $self->form->localize( $item->{label_loc} )
-            if defined $item->{label_loc};
-        $group{attributes} = $item->{attributes} || {};
-
-        return \%group;
+        $item->{group} = \@new;
     }
 
     if ( !exists $item->{attributes} ) {
@@ -178,8 +200,19 @@ sub _parse_option_hashref {
         }
     }
 
-    $item->{label} = $self->form->localize( $item->{label_loc} )
-        if defined $item->{label_loc};
+    if ( defined $item->{label_xml} ) {
+        $item->{label} = literal( $item->{label_xml} );
+    }
+    elsif ( defined $item->{label_loc} ) {
+        $item->{label} = $self->form->localize( $item->{label_loc} );
+    }
+
+    if ( defined $item->{value_xml} ) {
+        $item->{value} = literal( $item->{value_xml} );
+    }
+    elsif ( defined $item->{value_loc} ) {
+        $item->{value} = $self->form->localize( $item->{value_loc} );
+    }
 
     $item->{value} = '' if !defined $item->{value};
 
@@ -197,10 +230,6 @@ sub values {
         croak "values argument must be an array-ref" if $@;
     }
 
-    if ( $self->empty_first ) {
-        unshift @values, '';
-    }
-
     @new = (
         map { { value            => $_,
                 label            => ucfirst $_,
@@ -209,6 +238,9 @@ sub values {
             }
             } @values
     );
+    if ( $self->empty_first ) {
+        unshift @new, $self->_get_empty_first_option;
+    }
 
     $self->_options( \@new );
 
@@ -231,10 +263,6 @@ sub value_range {
     my $end   = pop @values;
     my $start = pop @values;
 
-    if ( $self->empty_first ) {
-        unshift @values, '';
-    }
-
     return $self->values( [ @values, $start .. $end ] );
 }
 
@@ -271,9 +299,23 @@ sub render_data_non_recursive {
             options => dclone( $self->_options ),
             @_ ? %{ $_[0] } : () } );
 
+    $self->_quote_options( $render->{options} );
+
     return $render;
 }
 
+sub _quote_options {
+    my ( $self, $options ) = @_;
+
+    foreach my $opt (@$options) {
+        $opt->{label} = xml_escape( $opt->{label} );
+        $opt->{value} = xml_escape( $opt->{value} );
+
+        $self->_quote_options( $opt->{group} )
+            if exists $opt->{group};
+    }
+}
+
 sub string {
     my ( $self, $args ) = @_;
 
@@ -337,6 +379,8 @@ L<HTML::FormFu::Element::Select> fields.
 
 =head2 options
 
+Arguments: none
+
 Arguments: \@options
 
     ---
@@ -352,6 +396,8 @@ Arguments: \@options
               style: highlighted
           - [ 04, April ]
 
+If passed no arguments, it returns an arrayref of the currently set options.
+
 Use to set the list of items in the select menu / radiogroup.
 
 It's arguments must be an array-ref of items. Each item may be an array ref 
@@ -380,8 +426,10 @@ An example of Select optgroups:
               - [2a, 'item 2a']
               - [2b, 'item 2b']
 
-The usage of label_loc instead of label is supported to translate a given
-string. label_loc is supported for items and option groups.
+When using the hash-ref construct, the C<label_xml> and C<label_loc> 
+variants of C<label> are supported, as are the C<value_xml> and C<value_loc> 
+variants of C<value>, the C<attributes_xml> variant of C<attributes> and the 
+C<label_attributes_xml> variant of C<label_attributes>.
 
 =head2 values
 
@@ -424,7 +472,17 @@ in select menus.
 
 If true, then a blank option will be inserted at the start of the option list
 (regardless of whether L</options>, L</values> or L</value_range> was used to
-populate the options).
+populate the options).  See also L</empty_first_label>.
+
+=head2 empty_first_label
+
+=head2 empty_first_label_xml
+
+=head2 empty_first_label_loc
+
+If L</empty_first> is true, and C<empty_first_label> is set, this value will
+be used as the label for the first option - so only the first option's value
+will be empty.
 
 =head1 SEE ALSO
 
